package dbpopulator;


import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;


public class DBPopulatorMySQL {
	private static Logger logger = Logger.getLogger(DBPopulatorMySQL.class.getName());
	

	private static final char[] ALFABETO={
		'a','b','c','d','e','f','g','h','i','j','k','l','m',
		'n','o','p','q','r','s','t','u','v','x','y','w','z',
		' ',
		};
	
	private static final int STRINGS_LENGTH = 20;
	private static final int NUM_TUPLES = 100;
	
	
	private static final String DB_DRIVER = "com.mysql.jdbc.Driver";
	private static final String DB_URI = "jdbc:mysql://localhost/";
	private static final String DB_NAME = "proj5";
	private static final String DB_USER = "root";
	private static final String DB_PASS = "root";	
	
	public static final Map<String, Integer> SCHEMA = new HashMap<String, Integer>() {
		private static final long serialVersionUID = 1L;

	{
		//	  relation	  num_attributes (exluding id field)
	    put(	"A",			6			);
	    put(	"B",			9			);
	    put(	"C",			8			);
	    put(	"D",			12			);
	    put(	"E",			11			);
	    put(	"F",			18			);
	    put(	"G",			12			);
	    put(	"H",			7			);
	    put(	"I",			12			);
	    put(	"L",			12			);
	    put(	"M",			17			);
	    put(	"N",			6			);
	    put(	"O",			16			);
	    put(	"P",			10			);
	    put(	"Q",			8			);
	    put(	"R",			16			);
	    put(	"S",			12			);
	    put(	"T",			9			);
	    put(	"U",			6			);
	    put(	"V",			9			);
	    put(	"Z",			11			);
	}};


	
	
	public boolean populateDB(){
		boolean res = true;
		
		this.createDatabase();
		
		logger.info("Populating database...");
		Set<String> relations = SCHEMA.keySet();
		for (Iterator iter = relations.iterator(); iter.hasNext();) {
			String rel = (String) iter.next();
			res &= this.createRelation(rel);
			res &= this.populateRelation(rel, NUM_TUPLES);
		}
		return res;
	}
	
	
	public boolean dropDatabase(){
		
		Connection conn = this.getConnection("");
		if( conn == null ) return false;
		
		logger.info("delete database...");
		String query = "DROP DATABASE "+ DB_NAME;
		try {
			Statement statement = conn.createStatement();
			statement.execute(query);
			statement.close();
		} catch (SQLException e) {
			logger.severe("errore in cleanDB(): " + e);
			try {conn.close();}
			catch (SQLException e2) {logger.severe("error closing connection in cleanDB(): "+ e2);}
			return false;
		}			
		try {conn.close();}
		catch (SQLException e) {logger.severe("error closing connection in cleanDB(): "+ e);}
		return true;	
	}

	
	private boolean populateRelation(String relation, int numTuple){
		if((SCHEMA.get(relation) == null) || (numTuple<=0)){
			logger.severe("populateRelation: bad parameters");
			return false;
		}
		
		Connection conn = this.getConnection(DB_NAME);
		int numAttr = SCHEMA.get(relation);
		boolean res = true;
		
		logger.info("Populate relation "+relation+" with "+numTuple+" tuples.");
		for(int i=0; i< numTuple; i++){
			StringBuffer query = new StringBuffer("INSERT INTO "+relation+" VALUES("+i);
			for(int j=1; j<=numAttr; j++)
				query.append(", '"+this.randomString()+"'");
			query.append(")");
			
			try {
				Statement statement = conn.createStatement();
				res &= statement.execute(query.toString());
				statement.close();
			} catch (SQLException e) {
				logger.severe("error in populateRelation(): "+ e);
				try {conn.close();}
				catch (SQLException e2) {logger.severe("error closing connection in populateRelation(): "+ e2);}
				return false;
			}
		}
		try {conn.close();}
		catch (SQLException e) {logger.severe("error closing connection in populateRelation(): "+ e);}
		return res;
	}
	
	
	public boolean createRelation(String relation){
		if(SCHEMA.get(relation) == null){
			logger.severe("createRelation: bad parameters");
			return false;
		}
		
		Connection conn = this.getConnection(DB_NAME);
		int numAttr = SCHEMA.get(relation);
		boolean res = true;
		
		logger.info("Create relation "+relation);
		StringBuffer query = new StringBuffer("CREATE TABLE IF NOT EXISTS "+relation+" ( id int(11) NOT NULL ");
		for(int j=1; j<=numAttr; j++)
			query.append(", "+relation+""+j+" text NOT NULL");
		query.append(",  PRIMARY KEY  (id))");
		
		try {
			Statement statement = conn.createStatement();
			res &= statement.execute(query.toString());
			statement.close();
		} catch (SQLException e) {
			logger.severe("error in createRelation(): "+ e);
			try {conn.close();}
			catch (SQLException e2) {logger.severe("error closing connection in createRelation(): "+ e2);}
			return false;
		}

		try {conn.close();}
		catch (SQLException e) {logger.severe("error closing connection in createRelation(): "+ e);}
		return res;
	}

	
	private boolean createDatabase(){
		Connection conn = this.getConnection("");
		if( conn == null ) return false;
		
		logger.info("create database...");
		String query = "CREATE DATABASE "+ DB_NAME;
		try {
			Statement statement = conn.createStatement();
			statement.execute(query);
			statement.close();
		} catch (SQLException e) {
			logger.severe("errore in createSchema(): " + e);
			try {conn.close();}
			catch (SQLException e2) {logger.severe("error closing connection in createSchema(): "+ e2);}
			return false;
		}			
		try {conn.close();}
		catch (SQLException e) {logger.severe("error closing connection in createSchema(): "+ e);}	
		return true;
	}

	private Connection getConnection(String dbName) {
		Connection connection = null;
		try {
		    Class.forName(DB_DRIVER);
		    connection = DriverManager.getConnection(DB_URI+dbName, DB_USER, DB_PASS);
		} catch (Exception e) {
			logger.severe("errore nel metodo getConnection(): " + e);
		}
		return connection;
	}
	
	private String randomString(){
		StringBuffer s=new StringBuffer();
	
		//la dimensione della stringa va da 1 a 20
		long l= Math.round(Math.random()*(STRINGS_LENGTH-1)+1);
		
		for (int i=0; i<l; i++){
			char c= ALFABETO[(int)Math.round(Math.random()*(ALFABETO.length-1))];
			s.append(c);
		}
		return s.toString();
	}


	public static void main(String[] args) {
		
		
		DBPopulatorMySQL pop = new DBPopulatorMySQL();
		
		pop.dropDatabase();
		
		pop.populateDB();
	}

}
