package proj5.business;

import java.io.FileNotFoundException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import proj5.business.flyweight.LeafFactory;
import proj5.model.core.Component;
import proj5.model.core.Composite;
import proj5.model.core.Leaf;
import proj5.utils.xml.XmlNotWellFormedException;

public class TreeBuilder {
	
	private LeafFactory leafFactory = null;
	private RandomIdGenerator idGenerator = null;
	
	public TreeBuilder(){
		this.leafFactory = LeafFactory.getIstance();
		this.idGenerator = new SimpleRandomIdGenerator();		
	}

	public Component buildTree(Document doc, boolean flyweightEnabled) throws FileNotFoundException, XmlNotWellFormedException{
		Node root = doc.getElementsByTagName("composite").item(0);
		this.leafFactory.setFlyweightEnable(flyweightEnabled);
		return this.recursiveTreeBuilder(root);
	}

	private Component recursiveTreeBuilder(Node node) throws XmlNotWellFormedException {

		if (node.getNodeName().equals("leaf")) 
			return createLeaf(node);
		else if (node.getNodeName().equals("composite")) 
			return createComposite(node);
		else 
			throw new XmlNotWellFormedException("node not expected: " + node.getNodeName());
	}

	private Component createComposite(Node node) throws XmlNotWellFormedException{
		Component composite = new Composite();
		NodeList childs = node.getChildNodes();
		for (int i = 0; i < childs.getLength(); i++) {
			Node child = childs.item(i);
			if ((child.getNodeName().equals("leaf")) || (child.getNodeName().equals("composite")))
				composite.add(this.recursiveTreeBuilder(child));
		}
		return composite;
	}
	
	private Leaf createLeaf(Node node) throws XmlNotWellFormedException{
		NamedNodeMap attributes = node.getAttributes();		
		Node leafTypeAttr = attributes.getNamedItem("type");
		if(leafTypeAttr == null)
			throw new XmlNotWellFormedException("Attribute \"type\" not found in node "+ node);
		
		String leafType = leafTypeAttr.getNodeValue();		
		Node idAttr = attributes.getNamedItem("id");
		int id = 0;
		if((idAttr == null))
			id =this.idGenerator.generateRandomId(leafType);
		else
			id = Integer.valueOf(idAttr.getNodeValue());
		
		return this.leafFactory.getLeaf(leafType, id);
	}


}
