package proj5.persistence.dao.core;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;



public class DataSource {
    private static DataSource singleton;
    private String databaseURL="jdbc:postgresql://localhost/gcd";
    private String username="postgres";
    private String password="free";
    private String driver="org.postgresql.Driver";
    private ConnectionPool connectionPool;
    private int maxConnections;
    private int minConnections;
    private boolean waitIfBusy;
    
    private void caricaProprieta() throws DAOException {
        String nomeFile = "pool.properties";
//        URL url = this.getClass().getClassLoader().getResource(nomeFile);
//        // HACK: on W32 and Tomcat5.5 %20 has to be replaced with spaces
//        String path = url.getPath();
//        path = path.replaceAll("%20"," ");
        try {
            FileInputStream file = new FileInputStream(nomeFile);
            Properties proprieta = new Properties();
            proprieta.load(file);
            System.out.println("Proprietà: "+proprieta);
            this.driver = proprieta.getProperty("driver");
            this.databaseURL = proprieta.getProperty("databaseURL");
            this.username = proprieta.getProperty("username");
            this.password = proprieta.getProperty("password");
            this.maxConnections = Integer.parseInt(proprieta.getProperty("maxConnections"));
            this.minConnections = Integer.parseInt(proprieta.getProperty("minConnections"));
            this.waitIfBusy = Boolean.parseBoolean(proprieta.getProperty("waitIfBusy"));
        } catch (FileNotFoundException fnfe) {
            System.err.println(fnfe);
            throw new DAOException("File di configurazione inesistente");
        } catch (IOException ioe) {
            System.err.println(ioe);
            throw new DAOException("Impossibile caricare la configurazione");
        }
    }
    private DataSource()throws DAOException {
        try{
            caricaProprieta();
            connectionPool=new ConnectionPool(driver,databaseURL,username,password,minConnections,maxConnections,waitIfBusy);
        }
        catch(SQLException e){
            throw new DAOException(e.getMessage());
        }
        
    }
    public static synchronized DataSource getInstance() {
            if (singleton==null){
                try {
                    singleton=new DataSource();
                }
                catch (DAOException e){
                    System.err.println(" impossibile inizializzare il driver "+e.getMessage());
                }
            }
            return singleton;
    }
    public Connection getConnection() throws SQLException{
        Connection con=null;
        try{
        con=connectionPool.getConnection();
        }
        catch (SQLException sqle) {
            close(con);
            throw new SQLException("getConnection: " + sqle);
        }
        return con;
        
    }
    public void close (Connection con){
        if (con!= null)
            connectionPool.free(con);
        /* *try{
            if (con!= null)
                con.close();
        }
        catch (SQLException e){
            System.err.println(" impossibile chiudere connessione --- "+e.getMessage());
        }*/
    }
    public void close(Statement statement) {
        try {
            if (statement != null)
                statement.close();
        } catch (SQLException sqle){
            System.err.println(sqle);
        }
    }
    public void close(ResultSet resultSet) {
        try {
        if (resultSet != null)
            resultSet.close();
        } catch (SQLException sqle){
            System.err.println(sqle);
        }
    }
}
